%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.lang.proc).

-export([bump_reductions/1, demonitor/1, group_leader/0, info/1, info/2,
	 is_alive/1, link/1, monitor/2, pid_to_string/1, self/0, send/2,
	 send_nosuspend/2, send_nosuspend/3, set_flag/2, set_flag/3,
	 set_group_leader/2, signal/2, spawn/1, spawn/2, spawn/3,
	 spawn/4, spawn_link/1, spawn_link/2, spawn_link/3,
	 spawn_link/4, spawn_opt/1, spawn_opt/2, spawn_opt/3,
	 spawn_opt/4, spawn_opt/5, string_to_pid/1, unlink/1, yield/0]).

-import(erl.lang.list).


bump_reductions(Reds) ->
    erl.lang.proc:bump_reductions(Reds).	% Built-in function

demonitor(Ref) ->
    erl.lang.proc:demonitor(Ref).	% Built-in function

group_leader() ->
    erl.lang.proc:group_leader().	% Built-in function

info(Pid) ->
    erl.lang.proc:info(Pid).		% Built-in function

info(Pid, Item) ->
    erl.lang.proc:info(Pid, Item).	% Built-in function

is_alive(Pid) ->
    erl.lang.proc:is_alive(Pid).	% Built-in function

link(Pid) ->
    erl.lang.proc:link(Pid).		% Built-in function

monitor(Type, Item) ->
    erl.lang.proc:monitor(Type, Item).	% Built-in function

pid_to_string(Pid) ->
    erl.lang.proc:pid_to_string(Pid).	% Built-in function

self() ->
    erl.lang.proc:self().		% Built-in function

send(Dest, Value) ->
%%%     erl.lang.proc:send(Dest, Value).	% Built-in function
    erl.lang:send(Dest, Value). % FIXME: move send/2 to erl.lang.proc in Beam

send_nosuspend(Pid, Msg) ->
    erl.lang.proc:send_nosuspend(Pid, Msg).	% Built-in function

send_nosuspend(Pid, Msg, Opts) ->
    erl.lang.proc:send_nosuspend(Pid, Msg, Opts).	% Built-in function

set_flag(Flag, Value) ->
    erl.lang.proc:set_flag(Flag, Value).	% Built-in function

set_flag(Pid, Flag, Value) ->
    erl.lang.proc:set_flag(Pid, Flag, Value).	% Built-in function

set_group_leader(Pid, Leader) ->
    erl.lang.proc:set_group_leader(Pid, Leader).	% Built-in function

signal(Pid, Value) ->
    erl.lang.proc:signal(Pid, Value).		% Built-in function

spawn(Mod, Func, Args) ->
    erl.lang.proc:spawn(Mod, Func, Args).	% Built-in function

spawn_link(Mod, Func, Args) ->
    erl.lang.proc:spawn_link(Mod, Func, Args).	% Built-in function

%% @doc Do not use this function! For internal purposes only.
spawn_opt(Data) ->
    erl.lang.proc:spawn_opt(Data).	% Built-in function

%% @doc For debugging only!
string_to_pid(Pid) ->
    erl.lang.proc:string_to_pid(Pid).	% Built-in function

unlink(Pid) ->
    erl.lang.proc:unlink(Pid).		% Built-in function

yield() ->
%%%     erl.lang.proc:yield().		% Built-in function
    erlang:yield(). % FIXME: erl.lang.proc:yield/0 not handled by Beam



% Spawns with a fun

%% FIXME: spawns from fun should be built-ins - not use apply/2!

spawn(F) when is_function(F) ->
    spawn(erl.lang, apply, [F, []]);
spawn({M,F}=MF) when is_atom(M), is_atom(F) ->
    spawn(erl.lang, apply, [MF, []]);
spawn(F) ->
    erl.lang:error(badarg, [F]).

%% FIXME: spawning from fun on remote node needs better implementation!

spawn(N, F) when N == node() ->
    spawn(F);
spawn(N, F) when is_function(F) ->
    spawn(N, erl.lang, apply, [F, []]);
spawn(N, {M,F}=MF) when is_atom(M), is_atom(F) ->
    spawn(N, erl.lang, apply, [MF, []]);
spawn(N, F) ->
    erl.lang:error(badarg, [N, F]).

spawn_link(F) when is_function(F) ->
    spawn_link(erl.lang, apply, [F, []]);
spawn_link({M,F}=MF) when is_atom(M), is_atom(F) ->
    spawn_link(erl.lang, apply, [MF, []]);
spawn_link(F) ->
    erl.lang:error(badarg, [F]).

spawn_link(N, F) when N == node() ->
    spawn_link(F);
spawn_link(N, F) when is_function(F) ->
    spawn_link(N, erl.lang, apply, [F, []]);
spawn_link(N, {M,F}=MF) when is_atom(M), is_atom(F) ->
    spawn_link(N, erl.lang, apply, [MF, []]);
spawn_link(N, F) ->
    erl.lang:error(badarg, [N, F]).

spawn_opt(F, O) when is_function(F) ->
    spawn_opt(erl.lang, apply, [F, []], O);
spawn_opt({M,F}=MF, O) when is_atom(M), is_atom(F) ->
    spawn_opt(erl.lang, apply, [MF, []], O);
spawn_opt({M,F,A}, O) -> % For (undocumented) backward compatibility
    spawn_opt(M, F, A, O);
spawn_opt(F, O) ->
    erl.lang:error(badarg, [F, O]).

spawn_opt(N, F, O) when N == node() ->
    spawn_opt(F, O);
spawn_opt(N, F, O) when is_function(F) ->
    spawn_opt(N, erl.lang, apply, [F, []], O);
spawn_opt(N, {M,F}=MF, O) when is_atom(M), is_atom(F) ->
    spawn_opt(N, erl.lang, apply, [MF, []], O);
spawn_opt(N, F, O) ->
    erl.lang:error(badarg, [N, F, O]).


% Spawns with MFA

%% FIXME: erl.lang.proc should not be dependent on 'gen_server' module!

spawn(N,M,F,A) when N == node(), is_atom(M), is_atom(F), is_list(A) ->
    spawn(M,F,A);
spawn(N,M,F,A) when is_atom(N), is_atom(M), is_atom(F) ->
    case list:is_proper(A) of
	true ->
	    ok;
	false ->
	    erl.lang:error(badarg, [N, M, F, A])
    end,
    case catch gen_server:call({net_kernel,N},
			       {spawn,M,F,A,group_leader()},
			       infinity) of
	Pid when is_pid(Pid) ->
	    Pid;
	Error ->
	    case remote_spawn_error(Error, {no_link, N, M, F, A, []}) of
		{error, Reason} ->
		    erl.lang:error(Reason, [N, M, F, A]);
		{ok, Pid} ->
		    Pid
	    end
    end;
spawn(N,M,F,A) ->
    erl.lang:error(badarg, [N, M, F, A]).

spawn_link(N,M,F,A) when N == node(), is_atom(M), is_atom(F), is_list(A) ->
    spawn_link(M,F,A);
spawn_link(N,M,F,A) when is_atom(N), is_atom(M), is_atom(F) ->
    case list:is_proper(A) of
	true ->
	    ok;
	_ ->
	    erl.lang:error(badarg, [N, M, F, A])
    end,
    case catch gen_server:call({net_kernel,N},
			       {spawn_link,M,F,A,group_leader()},
			       infinity) of
	Pid when is_pid(Pid) ->
	    Pid;
	Error ->
	    case remote_spawn_error(Error, {link, N, M, F, A, []}) of
		{error, Reason} ->
		    erl.lang:error(Reason, [N, M, F, A]);
		{ok, Pid} ->
		    Pid
	    end
    end;
spawn_link(N,M,F,A) ->
    erl.lang:error(badarg, [N, M, F, A]).

spawn_opt(M, F, A, Opts) ->
    case catch erl.lang.proc:spawn_opt({M, F, A, Opts}) of
	Pid when is_pid(Pid) ->
	    Pid;
	{'EXIT', {Reason, _}} ->
	    erl.lang:error(Reason, [M,F,A,Opts])
    end.

spawn_opt(N,M,F,A,O) when N == node(), is_atom(M), is_atom(F),
			  is_list(A), is_list(O) ->
    spawn_opt(M,F,A,O);
spawn_opt(N,M,F,A,O) when is_atom(N), is_atom(M), is_atom(F) ->
    case list:is_proper(A) andalso list:is_proper(O) of
	true ->
	    ok;
	false ->
	    erl.lang:error(badarg, [N, M, F, A, O])
    end,
    {L,NO} = list:foldl(fun (link, {_, NewOpts}) ->
				{link, NewOpts};
			    (Opt, {LO, NewOpts}) ->
				{LO, [Opt|NewOpts]}
			end,
			{no_link,[]}, O),
    case catch gen_server:call({net_kernel,N},
			       {spawn_opt,M,F,A,NO,L,group_leader()},
			       infinity) of
	Pid when is_pid(Pid) ->
	    Pid;
	Error ->
	    case remote_spawn_error(Error, {L, N, M, F, A, NO}) of
		{error, Reason} ->
		    erl.lang:error(Reason, [N, M, F, A, O]);
		{ok, Pid} ->
		    Pid
	    end
    end;
spawn_opt(N,M,F,A,O) ->
    erl.lang:error(badarg, [N,M,F,A,O]).

remote_spawn_error({'EXIT', {{nodedown,N}, _}}, {L, N, M, F, A, O}) ->
    {Opts, LL} = case L == link of
		     true ->
			 {[link|O], [link]};
		     false ->
			 {O, []}
		 end,
    {ok, spawn_opt(fun () -> crasher(N,M,F,A,Opts,noconnection) end, LL)};
remote_spawn_error({'EXIT', {Reason, _}}, _) ->
    {error, Reason};
remote_spawn_error({'EXIT', Reason}, _) ->
    {error, Reason};
remote_spawn_error(Other, _) ->
    {error, Other}.
    
crasher(Node,Mod,Fun,Args,[],Reason) ->
    error_logger:error_msg("** Can not start ~w:~w,~w on ~w **~n",
			  [ Mod,Fun,Args,Node]),
    exit(Reason);
crasher(Node,Mod,Fun,Args,Opts,Reason) ->
    error_logger:error_msg("** Can not start ~w:~w,~w (~w) on ~w **~n",
			  [ Mod,Fun,Args,Opts,Node]),
    exit(Reason).
