%% -*- erlang-indent-level: 2 -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Copyright (c) 2000 by Erik Johansson.  All Rights Reserved 
%% Time-stamp: <02/05/13 14:57:54 happi>
%% ====================================================================
%%  Filename : 	hipe_icode_cleanup.erl
%%  Module   :	hipe_icode_cleanup
%%  Purpose  :  To turn calls to exit and fault into exit and fault isntructions.
%%  Notes    : 
%%  History  :	* 2000-11-07 Erik Johansson (happi@csd.uu.se): 
%%                     Created.
%%              * 2003-04-10 Erik Stenman (happi@home.se):
%%                     Removed entry-flagging for catch blocks.
%%              * 2003-11-07 Erik Stenman:
%%                     Turned this pass into an assertion.
%%                     Moved all the rewriting to beam_to_icode.
%%                     This pass is now deprecated and will 
%%                     be removed sooner or later.
%%
%%  CVS      :
%%              $Author: kostis $
%%              $Date: 2003/11/14 14:02:01 $
%%              $Revision: 1.7 $
%% ====================================================================
%%  Exports  :
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

-module(hipe_icode_cleanup).
-export([code/1]).

%% Deprecated -- just a safety check.
code(Icode) ->
  %%  {_LMin,LMax} = hipe_icode:icode_label_range(Icode),
  %%  hipe_gensym:set_label(icode,LMax+1),
  %%  {_VMin,VMax} = hipe_icode:icode_var_range(Icode),
  %%  hipe_gensym:set_var(icode,VMax+1),
  Code = hipe_icode:icode_code(Icode),
  _NewCode = cleanup_code(Code),
  %%  hipe_icode:icode_code_update(Icode,NewCode).
  Icode.

cleanup_code(Code) ->
  cleanup_code(Code, []).

%% Deprecated -- just a safety check.
cleanup_code([I|Is], _Acc) ->
  cleanup_code(Is, cleanup_instr(I));%% ++ Acc);
cleanup_code([], _Acc) -> []. %% lists:reverse(Acc).

%% Deprecated -- just a safety check.
cleanup_instr(I) ->
  case hipe_icode:type(I) of
    call ->
      call_to_primop(I);
    enter ->
      enter_to_primop(I);
    _Other -> [I]
  end.

call_to_primop(I) ->
  Fun = hipe_icode:call_fun(I), 
  Po = hipe_icode:call_type(I),
  Bif = hipe_bif:is_bif(Fun),
  case {Po,Bif} of
    {primop,false} -> io:format("~w is primop but not bif\n",[Fun]),exit(I);
    {primop,true} -> ok;
    {Other,true} -> io:format("~w is ~w and bif\n",[Fun,Other]),exit(I);
    _ -> ok
  end,
  case Fun of
    {erlang,exit,1} ->
      io:format("Fail in call ~w found\n",[Fun]),
      exit(I);
%%      Args = hipe_icode:call_args(I),
%%      [hipe_icode:mk_fail(Args, exit)];
    {erlang,throw,1} ->
      io:format("Fail in call ~w found\n",[Fun]),
      exit(I);
%%      Args = hipe_icode:call_args(I),
%%      [hipe_icode:mk_fail(Args, throw)];
    {erlang,fault,1} ->
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:call_args(I),
%%      [hipe_icode:mk_fail(Args, fault)];
    {erlang,fault,2} ->
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:call_args(I),
%%      [hipe_icode:mk_fail(Args, fault2)];
    _ -> 
      [I]
  end.

enter_to_primop(I) ->
  Fun = hipe_icode:enter_fun(I), 
  Po = hipe_icode:enter_type(I),
  Bif = hipe_bif:is_bif(Fun),
  case {Po,Bif} of
    {primop,false} -> io:format("~w is primop but not bif\n",[Fun]),exit(I);
    {primop,true} -> ok;
    {Other,true} -> io:format("~w is ~w and bif\n",[Fun,Other]),exit(I);
    _ -> ok
  end,
  case Fun of
    {erlang,exit,1} -> 
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:enter_args(I),
%%      [hipe_icode:mk_fail(Args, exit)];
    {erlang,throw,1} ->
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:enter_args(I),
%%      [hipe_icode:mk_fail(Args, throw)];
    {erlang,fault,1} -> 
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:enter_args(I),
%%      [hipe_icode:mk_fail(Args, fault)];
    {erlang,fault,2} ->
      io:format("Fail in call ~w found\n",[Fun]),exit(I);
%%      Args = hipe_icode:enter_args(I),
%%      [hipe_icode:mk_fail(Args, fault2)];
    _ ->
      [I]
  end.

