%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%
%% 

%% Path to the c-program.
-define(SERVERDIR, filename:nativename(
		     filename:join(code:priv_dir(odbc), "bin"))).

%% Name of the C program 
-define(SERVERPROG, "odbcserver").

%% Constats defining the command protocol between the erlang control
%% process and the port program. These constants must also be defined
%% in the same way in the port program.

-define(OPEN_CONNECTION,	1).
-define(CLOSE_CONNECTION,	2).
-define(COMMIT_TRANSACTION,	3).
-define(COMMIT,			4).
-define(ROLLBACK,		5).
-define(QUERY,			6).
-define(SELECT_COUNT,		7).
-define(SELECT_FIRST,		8).
-define(SELECT_LAST,		9).
-define(SELECT_NEXT,		10).
-define(SELECT_PREV,		11).
-define(SELECT,			12).
-define(SELECT_RELATIVE,	13).
-define(SELECT_ABSOLUTE,	14).
-define(SELECT_N_NEXT,		15).
-define(PARAM_QUERY,		16).
-define(SHUTDOWN,		17).
-define(LENGTH_INDICATOR_SIZE,	4).
-define(INT_VALUE,		1).
-define(STR_VALUE,		2).
-define(ON,			1).
-define(OFF,			2).
-define(DUMMY_OFFSET,		0).


%% Types of parameters given to param_query
-define(USER_SMALL_INT, 1).
-define(USER_INT, 2).
-define(USER_DECIMAL, 3).
-define(USER_NMERIC, 4).
-define(USER_CHAR, 5).
-define(USER_VARCHAR, 6).
-define(USER_FLOAT, 7).
-define(USER_REAL, 8).
-define(USER_DOUBLE, 9).

%% EXIT CODES
-define(EXIT_SUCCESS,		0). % As defined in c iso_stdlib
-define(EXIT_FAILURE,		1). % As defined in c iso_stdlib
-define(EXIT_ALLOC,		2).
-define(EXIT_ENV,		3).
-define(EXIT_CONNECTION,	4).
-define(EXIT_FREE,		5).
-define(EXIT_STDOUT,		6).
-define(EXIT_STDIN,		7).
-define(EXIT_BIN,		8).
-define(EXIT_THREAD,		9).
-define(EXIT_TYPE,		10).
-define(EXIT_PARAM_ARRAY,       11).
-define(EXIT_OLD_WINSOCK,	12).

%% Misc constants
-define(DEFAULT_TIMEOUT, infinity).
-define(STR_TERMINATOR, 0).
-define(MAX_SEQ_TIMEOUTS, 10).

%% Handling of C exit codes
-define(ENCODE_EXIT_FUN, 
	(fun(?EXIT_SUCCESS) ->
		 normal_exit;
	   (?EXIT_FAILURE) ->
		 abnormal_exit_probably_external_communication_problems;
	    (?EXIT_ALLOC) ->
		 memory_allocation_failed;
	    (?EXIT_ENV)  ->
		 setting_of_environment_attributes_failed;
	    (?EXIT_CONNECTION) ->
		 setting_of_connection_attributes_faild;
	    (?EXIT_FREE) ->
		 freeing_of_memory_failed;
	    (?EXIT_STDOUT) ->
		 sending_message_to_erlang_failed;
	    (?EXIT_STDIN) ->
		 receiving_message_from_erlang_failed;
	    (?EXIT_BIN) ->
		 retrieving_of_binary_data_failed;
	    (?EXIT_THREAD) ->
		 failed_to_create_thread;
	    (?EXIT_TYPE) ->
		 wrong_type_in_param_query;
	    (?EXIT_PARAM_ARRAY) ->
		 does_not_support_param_arrays;
	    (?EXIT_OLD_WINSOCK) ->
		 too_old_verion_of_winsock;
	    (Other) ->
		 killed
	 end)).

-define(PORT_EXIT_REASON(EXIT_STATUS), 
	?ENCODE_EXIT_FUN(EXIT_STATUS)).
